#version 330 core

layout (location = 0) out vec4 nearColor;
layout (location = 1) out vec4 farColor;  

in vec2 TexCoords;

uniform sampler2D renderedSceneTexture;
uniform sampler2D depthTexture;
uniform float nearPlane;
uniform float farPlane;

float linearizeDepth(float depth)
{
    float z = depth * 2.0 - 1.0; // Back to NDC 
    return (2.0 * nearPlane * farPlane) / (farPlane + nearPlane - z * (farPlane - nearPlane));
}

void main()
{	
	vec4 black = vec4( vec3(0.0f) , 1.0f);

	float depthValue = texture(depthTexture, TexCoords).r;
	depthValue = linearizeDepth(depthValue) / farPlane ;

	//depthValue = linearizeDepth(depthValue);  // or without this divide??
	if( depthValue < 0.1f)
	{
		nearColor = texture(renderedSceneTexture, TexCoords);
		farColor = texture(renderedSceneTexture, TexCoords) * 0.1f;
	}
	else if(depthValue >= 0.1f && depthValue < 0.12f)
	{
		nearColor = texture(renderedSceneTexture, TexCoords) * (2.0f/3.0f) * 1.4f;
		farColor = texture(renderedSceneTexture, TexCoords) * (1.0f/3.0f) * 1.3f;
	}
	else if(depthValue >= 0.12f && depthValue < 0.15f)
	{
		nearColor = texture(renderedSceneTexture, TexCoords) * 0.5f * 1.5f;
		farColor = texture(renderedSceneTexture, TexCoords) * 0.5f * 1.5f;
	}
	else if(depthValue >= 0.15f && depthValue < 0.17f)
	{
		nearColor = texture(renderedSceneTexture, TexCoords) * (1.0f/3.0f) * 1.5f;
		farColor = texture(renderedSceneTexture, TexCoords) * (2.0f/3.0f) * 1.5f;
	}
	else if(depthValue >= 0.17f && depthValue < 0.2f)
	{
		nearColor = texture(renderedSceneTexture, TexCoords) * (1.0f/4.0f) * 1.5f;
		farColor = texture(renderedSceneTexture, TexCoords) * (3.0f/4.0f) * 1.5f;
	}
	else
	{
		nearColor = texture(renderedSceneTexture, TexCoords) * 0.1f;
		farColor = texture(renderedSceneTexture, TexCoords) * 1.2f;
	}
}