#version 330 core

out vec4 FragmentColor;
in vec2 TexCoords;

uniform sampler2D depthTexture;
uniform sampler2D renderedScene;

uniform mat4 uInverseModelViewMat; // inverse model->view
uniform mat4 uPrevModelViewProj; // previous model->view->projection
uniform mat4 projectionMatrix;

uniform int numberOfSamples;
uniform float currentFPS;
uniform float targetFPS;

// Frustum definition values
uniform float nearPlane;
uniform float farPlane;
uniform float l; // left
uniform float r; // right
uniform float b; // bottom
uniform float t; // top

// Quad/texture size in pixels
uniform float textureWidth;
uniform float textureHeight;

// calculate position of fragment in view space
vec3 getViewCoordinatesOfPixel()
{
	// z in non linear range [0,1]
	float zPixel = texture(depthTexture, TexCoords).r;

	// conversion into NDC [-1,1]
	float zNDC = zPixel * 2.0 - 1.0;

	float zEye = projectionMatrix[3][2] / ( zNDC - projectionMatrix[2][2]);

	// Converting from pixel coordinates to NDC
	float xNDC = gl_FragCoord.x / textureWidth * 2.0 - 1.0;
	float yNDC = gl_FragCoord.y / textureHeight * 2.0 - 1.0;

	float xEye = - zEye * ( xNDC * ( r - l) + ( r + l) ) / (2.0 * nearPlane);
	float yEye = - zEye * ( yNDC * ( t - b) + ( t + b) ) / (2.0 * nearPlane);

	vec3 eyecoords = vec3(xEye, yEye, zEye);

	return eyecoords;
}

float linearizeDepth(float depth)
{
	return (2.0 * nearPlane) / (farPlane + nearPlane - depth * (farPlane - nearPlane));
}

void main() 
{
	mat4 currentToPreviousMat = uPrevModelViewProj * uInverseModelViewMat;

	// get current world space position:
	vec3 vViewRay = getViewCoordinatesOfPixel();
	vec3 current = vViewRay * linearizeDepth(texture(depthTexture, TexCoords).r);
	vec4 previous = currentToPreviousMat * vec4(current, 1.0f);
	previous.xyz /= previous.w;
	previous.xy = previous.xy * 0.5f + 0.5f;

	vec2 blurVec = previous.xy - TexCoords;
  
	float mblurScale = currentFPS / targetFPS;
  
	blurVec *= mblurScale;

  
	// perform blur:
	vec4 FragmentColor = texture(renderedScene, TexCoords);
	for (int i = 1; i < numberOfSamples; ++i) // <= here ??
	{
		// get offset in range [-0.5, 0.5]:
		vec2 offset = blurVec * ( float(i) / float(numberOfSamples - 1) - 0.5f);
  
		// sample & add to result:
		FragmentColor += texture(renderedScene, TexCoords + offset);
   }
 
   //FragmentColor /= float(numberOfSamples);
   FragmentColor = vec4( vec3( 1.0f) , 1.0f);
}