
#version 330 core

struct Material 
{
    sampler2D texture_diffuse;
	sampler2D texture_diffuse1;
    sampler2D texture_specular;
	sampler2D texture_specular1;
	vec3 diffuseColor;
	vec3 specularColor;
    float shininess;
}; 

struct DirectionalLight 
{
    vec3 direction;
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;
};

in vec3 FragmentPosition;
in vec3 Normal;
in vec2 TexCoords;

out vec4 glFragColor;

uniform vec3 cameraPosition;
uniform DirectionalLight directionalLight;
uniform Material material;
uniform bool hasTexture;

void main()
{    
    // calculate normals and view direction
    vec3 normal = normalize(Normal);
    vec3 viewDirection = normalize(cameraPosition - FragmentPosition);
    
	// calculate directional light now
	vec3 lightDirection = normalize(-directionalLight.direction);

    // diffuse part
    float diffuse = max(dot(normal, lightDirection), 0.0f);

    // specular part
    vec3 reflectDirection = reflect(-lightDirection, normal);
    float specular = pow(max(dot(viewDirection, reflectDirection), 0.0f), material.shininess);

    // put it together, include texture information
	vec3 ambientColor;
	vec3 diffuseColor;
	vec3 specularColor;

	if(hasTexture)
	{
		ambientColor = directionalLight.ambient   * vec3(texture(material.texture_diffuse1, TexCoords));
		diffuseColor = directionalLight.diffuse   * diffuse  * vec3(texture(material.texture_diffuse1, TexCoords));
		specularColor = directionalLight.specular * specular * vec3(texture(material.texture_specular1, TexCoords));
	}
	else
	{
		// material has no texture. use material's colors instead
		//ambientColor = directionalLight.ambient;
		diffuseColor = directionalLight.diffuse   * diffuse  * material.diffuseColor;
		specularColor = directionalLight.specular * specular * material.specularColor;
	}

    vec3 resultColor = (ambientColor + diffuseColor + specularColor);

    glFragColor = vec4(resultColor, 1.0f);
}

