
#version 330 core
layout (location = 0) in vec3 position;
layout (location = 1) in vec3 normal;
layout (location = 2) in vec2 textureCoordinates;

out VS_OUT 
{
    vec3 FragmentPosition;
    vec3 Normal;
    vec2 TexCoords;
    vec4 FragmentPositionInLightSpace;
} vertexShaderOutput;

uniform mat4 modelMatrix;
uniform mat4 inverseModelMatrix;
uniform mat4 viewMatrix;
uniform mat4 projectionMatrix;
uniform mat4 toLightSpaceMatrix;

void main()
{    
    vertexShaderOutput.FragmentPosition = vec3(modelMatrix * vec4(position, 1.0));
    vertexShaderOutput.Normal = mat3(transpose(inverseModelMatrix)) * normal;
    vertexShaderOutput.TexCoords = textureCoordinates;
    vertexShaderOutput.FragmentPositionInLightSpace = toLightSpaceMatrix * vec4(vertexShaderOutput.FragmentPosition, 1.0);
    gl_Position = projectionMatrix * viewMatrix * modelMatrix * vec4(position, 1.0);
}